/* Autogenerated by png2wx.pl on Fri Oct 30 12:50:49 2015 */
#ifndef ICONS_H
#define ICONS_H 1

#include <wx/wxprec.h>
#ifndef WX_PRECOMP
#	include <wx/wx.h>
#endif

extern void initialize_images(void);

extern wxBitmap *_img_chartscale;

#endif /* ICONS_H */
